/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes
#include <wx/config.h>

#include "mviacamapp.h"
#include "mainwin.h"
//#include "SDL.h"
#include <exception>
#include "exceptions.h"
#include "puredatawrapper.h"
#include "cameracontroller.h"
#include "midicontroller.h"

////@begin XPM images

////@end XPM images

#ifdef WIN32
#ifdef _DEBUG
#include "MemLeakDetect.h"
CMemLeakDetect gMemLeakDetect;
#endif
#endif

/*!
 * Application instance implementation
 */

////@begin implement app
IMPLEMENT_APP( MultimediaViacamApp )
////@end implement app


/*!
 * 
 */

IMPLEMENT_CLASS( MultimediaViacamApp, wxApp )


/*!
 * 
 */

BEGIN_EVENT_TABLE( MultimediaViacamApp, wxApp )

////@begin MultimediaViacamApp event table entries
////@end MultimediaViacamApp event table entries

END_EVENT_TABLE()


/*!
 * Constructor for MultimediaViacamApp
 */

MultimediaViacamApp::MultimediaViacamApp()
{
    Init();
}


/*!
 * Member initialisation
 */

void MultimediaViacamApp::Init()
{
////@beginMultimediaViacamApp member initialisation
////@end MultimediaViacamApp member initialisation
	m_locale= new wxLocale();
}

/*!
 * Initialisation for MultimediaViacamApp
 */
#define LANGUAGE_KEY _T("language") 
void MultimediaViacamApp::SetLanguage()
{
	int lang= wxLANGUAGE_DEFAULT;
	wxString language;
	const wxLanguageInfo *info= NULL;

	// Load language name
	wxConfigBase::Get()->Read(LANGUAGE_KEY, &language);
	if (language.Length()> 0)
	{
		// Check if valid
		info= wxLocale::FindLanguageInfo (language);
		if (info)
		{
			// Ok
			lang= info->Language;
		}		
	}

	

	m_locale->AddCatalogLookupPathPrefix(wxT("."));
	m_locale->Init(lang, wxLOCALE_CONV_ENCODING);
	m_locale->AddCatalog(wxT("sitplus"));
	m_locale->AddCatalog(wxT("wxwidgets"));	
}

bool MultimediaViacamApp::OnInit()
{    
////@begin MultimediaViacamApp initialisation
    // Remove the comment markers above and below this block
    // to make permanent changes to the code.

#if wxUSE_XPM
    wxImage::AddHandler(new wxXPMHandler);
#endif
#if wxUSE_LIBPNG
    wxImage::AddHandler(new wxPNGHandler);
#endif
#if wxUSE_LIBJPEG
    wxImage::AddHandler(new wxJPEGHandler);
#endif
#if wxUSE_GIF
    wxImage::AddHandler(new wxGIFHandler);
#endif
////@end MultimediaViacamApp initialisation
	wxConfig::Set(new wxConfig(_T("sitplus")));

	SetLanguage();

	wxFrame* frame= new MainWin (NULL);	
    
	// Init singletons

	// Init camera controller
	CCameraController::getInstance(wxConfig::Get(), frame);

	// Init MIDI controller
	CMIDIController::getInstance(wxConfig::Get());


	SetTopWindow(frame);
    frame->Show();

    return true;
}

int MultimediaViacamApp::OnRun()
{
	try
	{		
		return wxApp::OnRun();		
	}
	catch (BaseException& e)
	{
		wxMessageDialog dlg (NULL, e.getMessage(), _("Fatal Error - Aborting"));
		dlg.ShowModal();
	}
	catch (std::exception& e)
	{
        wxMessageDialog dlg (NULL, wxString::From8BitData(e.what()), _("Fatal Error - Aborting"));
		dlg.ShowModal();
	}

	return -1;
}

void MultimediaViacamApp::OnUnhandledException()
{
	wxMessageDialog dlg (NULL, _T("Unexpected error."), _("Fatal Error - Aborting"));
	dlg.ShowModal();
}

/*!
 * Cleanup for MultimediaViacamApp
 */

int MultimediaViacamApp::OnExit()
{    
	wxConfig::Get()->Flush();

	// Finish singletons

	// Destroy Pd Wrapper class
	CPureDataSimpleController::destroyInstance();
	//CPureDataWrapper::destroyInstance();	
	CCameraController::destroyInstance();
	CMIDIController::destroyInstance();

	delete m_locale;
////@begin MultimediaViacamApp cleanup
    return wxApp::OnExit();
////@end MultimediaViacamApp cleanup
}

