/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2008 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#include "camselect.h"
#include "exceptions.h"
#include <wx/utils.h>
#include <wx/msgdlg.h>
#include <wx/choicdlg.h>
#include <wx/intl.h>

#if defined(__WXMSW__)
#include "crvcamera_wdm.h"
#include ".\camselect.h"
#else
#include "crvcamera_cv.h"
#endif

CCamSelect::CCamSelect(void)
{
}

CCamSelect::~CCamSelect(void)
{
}

#if defined(WIN32)
#define CAM_CLASS CCameraWDM
#else 
#define CAM_CLASS CCameraCV
#endif

// return the number of selected camera, -1 if not detected/selected any camera
// name is updated accordingly
int CCamSelect::SelectCameraInteractive(wxString& name, const wxString message, bool force)
{
	CCamera* cam;
	int numDevices;
	int camId= -1;

	numDevices= CAM_CLASS::GetNumDevices ();
	if (numDevices== 0)
	{
		throw CameraException (_("Not detected any camera."));
		//wxMessageDialog errorMsg (NULL, _("Not detected any camera. Aborting"), message, wxOK | wxICON_ERROR);
		//errorMsg.ShowModal();
		//return -1;
	}	
	
	// Try to find previously used camera
	if (!force && name.Length()> 0)
	{
		for (camId= 0; camId< numDevices; camId++)
			if (wxString(CAM_CLASS::GetDeviceName (camId), wxConvLibc)== name) break;			
		if (camId== numDevices) camId= -1;	// Not found
	}

	// Show selection dialog when needed
	if (camId== -1)
	{
		wxArrayString strArray;

		for (camId= 0; camId< numDevices; camId++)
			strArray.Add (wxString(CAM_CLASS::GetDeviceName (camId), wxConvLibc));

		wxSingleChoiceDialog choiceDlg(NULL, _("Choose the camera to use"), message, strArray, 
							NULL, wxDEFAULT_DIALOG_STYLE | wxOK | wxCANCEL | wxCENTRE);

		if (choiceDlg.ShowModal ()!= wxID_OK) {
			name.Clear();
			throw CameraException (_("No camera has been selected."));
		}
			//return -1;

		camId= choiceDlg.GetSelection();
		name= choiceDlg.GetStringSelection ();
	}

	cam= new CAM_CLASS(camId, 320, 240);
	
	// Try to open the camera to ensure it works
	if (!cam->Open ())
	{
		delete cam;
		throw CameraException (_("Cannot initialize the camera.\nPerhaps is being used by other application."));

		//wxMessageDialog errorMsg (NULL, _("Can not initialize the camera.\nPerhaps is being used by other application."), message, wxOK | wxICON_ERROR);
		//errorMsg.ShowModal();
		//camId= -1;
	}
	else
		cam->Close();

	delete cam;

	return camId;
}

int CCamSelect::GetCamIdByName(const wxString& name)
{
	int numDevices;
	int camId= -1;

	numDevices= CAM_CLASS::GetNumDevices ();
	if (name.Length()> 0)
	{
		for (camId= 0; camId< numDevices; camId++)
			if (wxString(CAM_CLASS::GetDeviceName (camId), wxConvLibc)== name) break;			
		if (camId== numDevices) camId= -1;	// Not found
	}
	return camId;
}
