/////////////////////////////////////////////////////////////////////////////
// Name:        visiblenormroi2.h
// Purpose:  This class is used to implement ROI like objects which can
//           be showed through CCamWindow pannels and modified 
//			 interactively with the mouse. Inhetits CNormROI. The class is
//			 thread safe.				 
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2008-10 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#pragma once

#include "crvnormroi.h"
#include <wx/gdicmn.h>
#include <wx/thread.h>
#include <wx/event.h>

class CVisibleNormROI2 : protected CNormROI
{
public:
	CVisibleNormROI2(void);
	CVisibleNormROI2(float width, float height, float x= -1.0f, float y= -1.0f, bool visible= true);
	virtual ~CVisibleNormROI2(void);

	// Working with native coordinates (overriden)
	void SetP1Resize (const float x, const float y);
	void SetP1Move (const float x, const float y);
	void SetP2Resize (const float x, const float y);
	void SetCenter (const float x, const float y);
	void GetCenter (float& x, float& y);
	void SetSize (const float width, const float height);
	void GetSize (float& width, float& height);

	// These are overriden to support locking
	void SetP1ResizeImg (const CIplImage *pImg, const int x, const int y);
	void SetP1MoveImg (const CIplImage *pImg, const int x, const int y);
	void SetP2ResizeImg (const CIplImage *pImg, const int x, const int y);
	void SetCenterImg (const CIplImage *pImg, const int x, const int y);
	void GetCenterImg (const CIplImage *pImg, int& x, int& y);
	void SetSizeImg (const CIplImage *pImg, const int width, const int height);
	void GetBoxImg (const CIplImage *pImg, int& x, int& y, int& width, int& height);
	void GetBoxImg (const CIplImage *pImg, CvRect& box);

	float GetRotation () { return CNormROI::GetRotation(); }
	void SetRotation (float value) { CNormROI::SetRotation(value); }

	bool GetShowOrientation () const { return m_showOrientation; }
	void SetShowOrientation (const bool value) { m_showOrientation= value; }
		
	enum { OVER_NONE= 0, OVER_LEFT_LINE, OVER_BOTTOM_LINE, OVER_RIGHT_LINE, OVER_UPPER_LINE, 
		   OVER_ORIENTER, OVER_UL_CORNER, OVER_BR_CORNER };

	
	bool RegisterChildROI (CVisibleNormROI2* pNormROI);
	bool UnregisterChildROI (CVisibleNormROI2* pNormROI);
	
	//void Paint (const CvSize& winSize, CIplImage *pImg);
	void Paint (CIplImage& img, const wxWindow& win);
	// Return true if processed
	bool MouseEvent (wxMouseEvent& event, const wxPoint& prevCursor);

private:

	// Destruction
	virtual void Destroy();

	// Returns where is the cursor over
	int MouseMovedNoClick (const CvSize& winSize, const CvPoint& cursor);
	int MouseMovedNoClickRec (const CvSize& winSize, const CvPoint& cursor);
	int MouseMovedNoClick0 (const CvSize& winSize, const CvPoint& cursor);

	bool MouseMovedClick (const CvSize& winSize, const CvPoint& cursor, const CvPoint& prevCursor);
	bool MouseMovedClickRec (const CvSize& winSize, const CvPoint& cursor, const CvPoint& prevCursor);
	bool MouseMovedClick0 (const CvSize& winSize, const CvPoint& cursor, const CvPoint& prevCursor);
	
	
//	void OnPaint (const CvSize& winSize, CIplImage *pImg);


	void GetDirectionSegment (const CvSize& winSize, CvPoint& p1, CvPoint& p2);


	
	//void OnPaintRec (const CvSize& winSize, CIplImage *pImg);
	//void OnPaint0 (const CvSize& winSize, CIplImage *pImg);
	void PaintRec (const CvSize& winSize, CIplImage& img);
	void Paint0 (const CvSize& winSize, CIplImage& img);
	
	
	void Lock();
	void Unlock();
		
	// We use a global mutex object instead of the "lock on parent root" method
	// previously used to simplify the synchronization process.
	static wxCriticalSection g_CriticalSection;
	//wxCriticalSection m_CriticalSection;
	bool m_visible;
	int m_cursorOver;
	bool m_showOrientation;
};

